==> haystack-0.5.0%2B2.tar.gz: lib/src/io/HZincWriter.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hzinc_writer;

import '../HBin.dart';
import '../HGrid.dart';
import '../HList.dart';
import '../HMarker.dart';
import '../HVal.dart';
import '../HXStr.dart';
import 'HGridWriter.dart';

/// HZincWriter is used to write grids in the Zinc format
///
/// See [Project Haystack](http://project-haystack.org/doc/Zinc)
class HZincWriter extends HGridWriter {

  /// Write using UTF-8 
  factory HZincWriter(StringBuffer out) {
    return new HZincWriter._(out);
  }

  /// Write a grid to an in-memory a string 
  static String gridToString(HGrid grid) {
    StringBuffer out = new StringBuffer();
    new HZincWriter(out).writeGrid(grid);
    return out.toString();
  }

  static String valToString(HVal val) {
    var out = new StringBuffer();
    new HZincWriter(out).writeVal(val);
    return out.toString();
  }

  HZincWriter._(this._out);

  HZincWriter writeVal(HVal val) {
    if (val is HGrid) {
      HGrid grid = val;
      try {
        bool insideGrid = _gridDepth > 0;
        ++_gridDepth;
        if (insideGrid) _writeNestedGrid(grid);
        else writeGrid(grid);
      } finally {
        --_gridDepth;
      }
    } else if (val is HList) _writeList(val);
    else if (val is HDict) _writeDict(val);
    else _writeScalar(val);
    return this;
  }

  void _writeNestedGrid(HGrid grid) {
    _p("<<")._nl();
    writeGrid(grid);
    _p(">>");
  }


  void _writeList(HList list) {
    _p('[');

    for (int i=0; i<list.length; ++i) {
      if (i > 0) _p(',');
      writeVal(list.get(i));
    }
    _p(']');
  }

  void _writeDict(HDict dict) {
    _p('{')._writeMeta(dict)._p('}');
  }

  void _writeScalar(HVal val) {
    if (val == null) _p('N');
    else if (val is HBin) _writeBin(val);
    else if (val is HXStr) _writeXStr(val);
    else _p(val.toZinc());
  }

  void _writeBin(HBin bin)
  {
    if (this._version < 3) {
      _out..write("Bin(")..write(bin.mime)..write(')');
    } else {
      _p(bin.toZinc());
      _out..write('Bin("')..write(bin.mime)..write('")');
    }
  }


  void _writeXStr(HXStr xstr) {
    if (this._version < 3)
      throw new StateError("XStr not supported for version: $_version");
    _p(xstr.toZinc());
  }

  /// Write a grid 
  @override
  void writeGrid(HGrid grid) {
    // meta
    _out.write("ver:\"$_version.0\"");
    _writeMeta(grid.meta);
    _out.write('\n');

    // cols
    for ( int i = 0; i < grid.numCols; ++i ) {
      if (i > 0) _out.write(',');
      _writeCol(grid.colByIndex(i));
    }
    _out.write('\n');

    // rows
    for (int i=0; i<grid.numRows; ++i) {
      _writeRow(grid, grid.row(i));
      _out.write('\n');
    }
  }

  HZincWriter _writeMeta(HDict meta) {
    if (meta.isEmpty) return this;

    for (var it = meta.iterator(); it.moveNext(); ) {
      DictEntry entry = it.current;
      HVal val = entry.val;
      _out..write(' ')..write(entry.key);

      if (val != HMarker.VAL) {
        _out..write(':')..write(val.toZinc());
      }
    }

    return this;
  }

  void _writeCol(HCol col) {
    _out.write(col.name);
    _writeMeta(col.meta);
  }

  void _writeRow(HGrid grid, HRow row) {
    for (int i=0; i<grid.numCols; ++i) {
      HVal val = row.getByCol(grid.colByIndex(i), checked:false);
      if (i > 0) _out.write(',');
      if (val == null) {
        if (i == 0) _out.write('N');
      } else if (val is HGrid) {
        _out.write('<<\n');
        val.dump(_out);
        _out.write('\n>>');
      } else {
        _out.write(val.toZinc());
      }
    }
  }

  HZincWriter _p(Object obj) { _out.write(obj); return this; }
  HZincWriter _nl() { _out.write('\n'); return this; }
//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  int _version = 3;
  int _gridDepth = 0;
  StringBuffer _out;
}

==> haystack-1.0.0-dev.1.0%2B2.tar.gz: lib/src/io/HZincWriter.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hzinc_writer;

import '../HBin.dart';
import '../HGrid.dart';
import '../HList.dart';
import '../HMarker.dart';
import '../HVal.dart';
import '../HXStr.dart';
import 'HGridWriter.dart';

/// HZincWriter is used to write grids in the Zinc format
///
/// See [Project Haystack](http://project-haystack.org/doc/Zinc)
class HZincWriter extends HGridWriter {
  /// Write using UTF-8
  factory HZincWriter(StringBuffer out) {
    return new HZincWriter._(out);
  }

  /// Write a grid to an in-memory a string
  static String gridToString(HGrid grid) {
    StringBuffer out = new StringBuffer();
    new HZincWriter(out).writeGrid(grid);
    return out.toString();
  }

  static String valToString(HVal val) {
    var out = new StringBuffer();
    new HZincWriter(out).writeVal(val);
    return out.toString();
  }

  HZincWriter._(this._out);

  HZincWriter writeVal(HVal? val) {
    if (val is HGrid) {
      HGrid grid = val;
      try {
        bool insideGrid = _gridDepth > 0;
        ++_gridDepth;
        if (insideGrid) {
          _writeNestedGrid(grid);
        } else {
          writeGrid(grid);
        }
      } finally {
        --_gridDepth;
      }
    } else if (val is HList) {
      _writeList(val);
    } else if (val is HDict) {
      _writeDict(val);
    } else {
      _writeScalar(val);
    }
    return this;
  }

  void _writeNestedGrid(HGrid grid) {
    _p("<<")._nl();
    writeGrid(grid);
    _p(">>");
  }

  void _writeList(HList list) {
    _p('[');

    for (int i = 0; i < list.length; ++i) {
      if (i > 0) {
        _p(',');
      }
      writeVal(list.get(i));
    }
    _p(']');
  }

  void _writeDict(HDict dict) {
    _p('{')._writeMeta(dict)._p('}');
  }

  void _writeScalar(HVal? val) {
    if (val == null) {
      _p('N');
    } else if (val is HBin) {
      _writeBin(val);
    } else if (val is HXStr) {
      _writeXStr(val);
    } else {
      _p(val.toZinc());
    }
  }

  void _writeBin(HBin bin) {
    if (this._version < 3) {
      _out..write("Bin(")..write(bin.mime)..write(')');
    } else {
      _p(bin.toZinc());
      _out..write('Bin("')..write(bin.mime)..write('")');
    }
  }

  void _writeXStr(HXStr xstr) {
    if (this._version < 3) {
      throw new StateError("XStr not supported for version: $_version");
    }
    _p(xstr.toZinc());
  }

  /// Write a grid
  @override
  void writeGrid(HGrid grid) {
    // meta
    _out.write("ver:\"$_version.0\"");
    _writeMeta(grid.meta);
    _out.write('\n');

    // cols
    for (int i = 0; i < grid.numCols; ++i) {
      if (i > 0) {
        _out.write(',');
      }
      _writeCol(grid.colByIndex(i));
    }
    _out.write('\n');

    // rows
    for (int i = 0; i < grid.numRows; ++i) {
      _writeRow(grid, grid.row(i));
      _out.write('\n');
    }
  }

  HZincWriter _writeMeta(HDict meta) {
    if (meta.isEmpty) return this;


    meta.forEach((key, value) {
      _out..write(' ')
        ..write(key);

      if (value != HMarker.VAL) {
        _out..write(':')..write(value.toZinc());
      }
    });

    return this;
  }

  void _writeCol(HCol col) {
    _out.write(col.name);
    _writeMeta(col.meta);
  }

  void _writeRow(HGrid grid, HRow row) {
    for (int i = 0; i < grid.numCols; ++i) {
      HVal? val = row.getByCol(grid.colByIndex(i), checked: false);
      if (i > 0) {
        _out.write(',');
      }
      if (val == null) {
        if (i == 0) {
          _out.write('N');
        }
      } else if (val is HGrid) {
        _out.write('<<\n');
        val.dump(_out);
        _out.write('\n>>');
      } else {
        _out.write(val.toZinc());
      }
    }
  }

  HZincWriter _p(Object obj) {
    _out.write(obj);
    return this;
  }

  HZincWriter _nl() {
    _out.write('\n');
    return this;
  }
//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  int _version = 3;
  int _gridDepth = 0;
  StringBuffer _out;
}
